@@ -1,7 +1,7 @@
 /*
  * $Id: par.c 4910 2014-01-16 20:12:45Z heas $
  *
- * Copyright (C) 2002-2016 by Henry Kilmer, Andrew Partan, John Heasley
+ * Copyright (C) 2002-2017 by Henry Kilmer, Andrew Partan, John Heasley
  * All rights reserved.
  *
  * This software may be freely copied, modified and redistributed without
@@ -63,7 +63,6 @@
 # include <strings.h>
 #endif
 #include <sys/time.h>
-#include <sys/time.h>
 #ifdef HAVE_WAIT_H
 # include <wait.h>
 #elif HAVE_SYS_WAIT_H
@@ -100,7 +99,9 @@ int		debug = 0,
 int		devnull;			/* /dev/null */
 
 /* args */
-int		e_opt = 0,
+int		F_opt = 0,
+		H_opt = 0,
+		e_opt = 0,
 		f_opt = 0,
 		i_opt = 0,
 		n_opt = 3,
@@ -119,7 +120,7 @@ void		arg_free(char ***);
 int		arg_mash(char **, char **);
 int		arg_replace(char **, char **, char **, char ***);
 #ifndef HAVE_ASPRINTF
-int		asprintf(char **, char const *, ...)
+int		asprintf(char **, char const *, ...);
 #endif
 int		dispatch_cmd(char **, char **);
 int		execcmd(child *, char **);
@@ -139,8 +140,6 @@ RETSIGTYPE	reapchild(int);
 int
 main(int argc, char **argv, char **envp)
 {
-    extern char		*optarg;
-    extern int		optind;
     time_t		t;
     int			i,
 			line;
@@ -169,8 +168,14 @@ main(int argc, char **argv, char **envp)
     } else
 	errfp = stderr;
 
-    while ((i = getopt(argc, argv, "defhiqxvc:e:l:n:p:")) != -1 )
+    while ((i = getopt(argc, argv, "FHdefhiqxvc:e:l:n:p:")) != -1 )
 	switch (i) {
+	case 'F':
+	    F_opt = 1;
+	    break;
+	case 'H':
+	    H_opt = 1;
+	    break;
 	case 'c':	/* command to run */
 	    c_opt = optarg;
 	    break;
@@ -665,24 +670,52 @@ arg_replace(char **cmd, char **args, char **tail, char ***new)
 #ifndef HAVE_ASPRINTF
 /* crude emulation of asprintf() */
 int
-asprintf(char **str, char const *fmt, va_list ap)
+asprintf(char **ret, const char *format, ...)
 {
-    int len;
-    va_list apc;
+    va_list	ap;
+    char	*newbuf;
+    int		len,
+		vlen;
 
-    va_copy(apc, ap);
-    len = vsnprintf(NULL, 0, fmt, apc);
-    va_end(apc);
-    if (len < 0)
-	return len;
-    if ((*str = malloc(len + 1)) == NULL)
-	return -1;
-    if ((len = vsnprintf(*str, len + 1, fmt, ap)) >= 0)
-	return len;
-    len = errno;
-    free(*str);
-    errno = len;
-    return -1;
+    if (ret == NULL) {
+	errno = EINVAL;
+	return(-1);
+    }
+
+    if (format == NULL) {
+	*ret = NULL;
+	return(0);
+    }
+
+    va_start(ap, format);
+    len = strlen(format) + 1;
+    if (ap != NULL)
+	len = len * 2;
+    va_end(ap);
+
+    if ((*ret = (char *)malloc(len)) == NULL) {
+	errno = ENOMEM;
+	*ret = NULL;
+	return(-1);
+    }
+
+    while (1) {
+	va_start(ap, format);
+	if ((vlen = vsnprintf(*ret, len, format, ap)) < len &&
+	    (vlen != -1)) {
+	    va_end(ap);
+	    return(vlen);
+	}
+	va_end(ap);
+	len = vlen + 1;
+	if ((newbuf = realloc((void *)*ret, len)) == NULL) {
+	    free(*ret);
+	    *ret = NULL;
+	    errno = ENOMEM;
+	    return(-1);
+	}
+	*ret = newbuf;
+    }
 }
 #endif
 
@@ -742,7 +775,7 @@ execcmd(child *c, char **cmd)
     /* block sigchld so we quickly reap it ourselves */
     sigprocmask(SIG_BLOCK, &set_chld, NULL);
 
-    if (c->logfile) {
+    if (c->logfile && !H_opt) {
 	char	*ct;
 	t = time(NULL);
 		/* XXX: build a complete cmd line */
@@ -1129,7 +1162,7 @@ shcmd(child *c, char **cmd)
     /* block sigchld so we quickly reap it ourselves */
     sigprocmask(SIG_BLOCK, &set_chld, NULL);
 
-    if (c->logfile) {
+    if (c->logfile && !H_opt) {
 	char	*ct;
 	t = time(NULL);
 	ct = ctime(&t); ct[strlen(ct) - 1] = '\0';
@@ -1205,7 +1238,7 @@ xtermcmd(child *c, char **cmd)
     /* block sigchld so we quickly reap it ourselves */
     sigprocmask(SIG_BLOCK, &set_chld, NULL);
 
-    if (c->logfile) {
+    if (c->logfile && !H_opt) {
 	char	*ct;
 	t = time(NULL);
 		/* XXX: build a complete cmd line */
@@ -1371,7 +1404,7 @@ reapchild(int sig)
 			fprintf(errfp, "%d finished (logfile %s)\n",
 				(uint32_t)pid, progeny[i].logfname);
 		}
-		if (progeny[i].logfile != NULL) {
+		if (progeny[i].logfile != NULL && !F_opt) {
 		    str = ctime(&t); str[strlen(str) - 1] = '\0';
 		    fprintf(progeny[i].logfile, "Ending: %s: pid = %d\n",
 			    str, (uint32_t)pid);
@@ -1402,8 +1435,8 @@ reapchild(int sig)
 void
 usage(void)
 {
-    fprintf(errfp, "usage: %s [-dfiqvx] [-n #] [-p n] [-l logfile] [-c "
-		   "command] [<command file>]\n", progname);
+    fprintf(errfp, "usage: %s [-FHdfiqvx] [-n #] [-p n] [-l logfile] "
+		   "[-c command] [<command file>]\n", progname);
     return;
 }
 
